#
#  APRS4R - a ruby based aprs gateway/digipeater
#  Copyright (C) 2006 by Michael Conrad <do5mc@friggleware.de>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
#

require 'Logger'

require 'aprs4r/APRSMessage'
require 'aprs4r/Plugin'
require 'aprs4r/XMLDataPluginConfiguration'


class XMLDataPlugin < Plugin

  @@logger = Logger.get_logger( "XMLDataPlugin")

  
  def initialize( configuration)
    @@logger.info( "initialize( configuration)")

    super( configuration)

    @file = configuration.file
    @device = configuration.device

    @call = configuration.call
    @latitude = configuration.latitude
    @longitude = configuration.longitude

    # use default value if timeout is missed
    @timeout = 1800
    @timeout = configuration.timeout if configuration.timeout

    # use default value if period is missed
    @period = 15
    @period = configuration.period if configuration.period


    @timestamps = Hash.new
    @messages = Hash.new

    @symbols = Hash.new
    @positions = Hash.new
    @comments = Hash.new
    @states = Hash.new
    @weathers = Hash.new
    @courses = Hash.new
    @phgs = Hash.new
    
    return
  end

  
  def start 
    @@logger.info( "start")

    if ! @enable
      return
    end

    registerListener

    Thread.new { run() } 

    return
  end

  
  def registerListener
    @@logger.info( "registerListener")

    if @enable
      SocketManager.addListener( @device, self)
    end

    return
  end


  def unregisterListener
    @@logger.info( "registerListener")

    if @enable
      SocketManager.removeListener( @device, self)
    end

    return
  end


  def receiveAPRSMessage( name, message)
    @@logger.info( "receiveAPRSMessage( name, message)")

    if message.nil? 
      return
    end

    source = message.source.to_s

    now = Time.now
    @timestamps[source] = now
    @messages[source] = message

    if message.has_symbol? 
      @symbols[source] = message
      # @@logger.warn( "message: symbol: #{message.symbol_code} name: #{message.symbol_name}")
    end

    if message.has_position?
      @positions[source] = message
      # @@logger.warn( "message: lat: #{message.latitude} long: #{message.longitude}")
    end

    if message.has_comment?
      @comments[source] = message
      # @@logger.warn( "message: comment: #{message.comment}")
    end

    if message.has_status?
      @states[source] = message
      # @@logger.warn( "message: status: #{message.status}")
    end

    if message.has_weather?
      @weathers[source] = message
      # @@logger.warn( "message: weather: wind: direction: #{message.weather_wind_direction} speed: #{message.weather_wind_speed} temperature: #{message.weather_temperature} humidity: #{message.weather_humidity} pressure: #{message.weather_pressure} rain: #{message.weather_rain}")
    end

    if message.has_course?
      @courses[source] = message
      # @@logger.warn( "message: course: direction: #{message.course_direction} speed: #{message.course_speed}")
    end

    if message.has_phg?
      @phgs[source] = message
      # @@logger.warn( "message: phg: power: #{message.phg_power} height: #{message.phg_height} gain: #{message.phg_gain} direction: #{message.phg_direction}")
    end

    return
  end


  def run 
    @@logger.info( "run")

    while true 

      now = Time.now

      # clean up station hash
      @timestamps.each{ |key, timestamp|
        if now - timestamp > @timeout
          @timestamps.delete( key)

          @messages.delete( key)

          @symbols.delete( key)
          @positions.delete( key)
          @comments.delete( key)
          @states.delete( key)
          @weathers.delete( key)
          @courses.delete( key)
          @phgs.delete( key)
        end
      }

      file = File.new( @file, "w")

      # write xml header
      file.puts( "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>")

      # write stations element
      stations = sprintf( "<stations call=\"%s\" lat=\"%02.04f\" lng=\"%03.04f\">", @call, @latitude, @longitude)
      file.puts( stations)

      # write station element for each station
      @messages.each{ |key, message|

        if message
          source = message.source
          destination = message.destination
          path = message.path

          path_list = String.new
          for i in 0...path.length
            if i != 0 
              path_list += ","
            end

            path_list += path[i].to_s
          end

          timestamp = @timestamps[source]

          # start station section
          type = "else"
          type = message.symbol_name if message.has_symbol? && message.symbol_name != ""
          # @@logger.warn( "symbol: code: #{message.symbol_code} name: #{message.symbol_name}")
          station_open = sprintf( "<station source=\"%s\" destination=\"%s\" path=\"%s\" name=\"%s\">", source, destination, path_list, type)
          file.puts( station_open)

          # write position
          position = @positions[source]

          output = "<position lat=\"00.0000\" lng=\"000.0000\"/>"
          output = sprintf( "<position lat=\"%02.04f\" lng=\"%03.04f\"/>", position.latitude, position.longitude) if position && position.has_position?
          file.puts( output)
                                
          # write timestamp
          timestamp = @timestamps[source]
          if timestamp
            output = sprintf( "<time year=\"%04.4i\" month=\"%02.2i\" day=\"%02.2i\" hour=\"%02.2i\" minute=\"%02.2i\"/>", now.year, now.month, now.day, now.hour, now.min)
            file.puts( output)
          end

          # write comment section
          comment = @comments[source]
          if comment
            output = sprintf( "<comment>%s</comment>", escapeHTML( comment.comment)) if comment.has_comment?
            file.puts( output)
          end

          # write status section
          status = @states[source]
          if status
            output = sprintf( "<status>%s</status>", escapeHTML( status.status)) if status.has_status?
            file.puts( output)
          end

          # write course section
          course = @courses[source]
          if course 
            output = sprintf( "<course direction=\"%i\" speed=\"%i\"/>", course.course_direction, course.course_speed) if course.has_course?
            file.puts( output)
          end

          # write phg section
          phg = @phgs[source]
          if phg 
            output = sprintf( "<phg pwr=\"%i\" hgt=\"%i\" gn=\"%i\" dr=\"%s\"/>", phg.phg_power, phg.phg_height, phg.phg_gain, phg.phg_direction) if phg.has_phg?
            file.puts( output)
          end

          # write weather section
          weather = @weathers[source]
          if weather
            output = sprintf( "<weather windspeed=\"%i\" winddirection=\"%i\" humid=\"%i\" temperature=\"%i\" pressure=\"%f\" rain=\"%i\"/>", weather.weather_wind_speed, weather.weather_wind_direction, weather.weather_humidity, weather.weather_temperature, weather.weather_pressure, weather.weather_rain) if weather.has_weather?
            file.puts( output)
          end

          # end station section
          file.puts( "</station>")
        end
      }

      stations = "</stations>"
      file.puts( stations)

      file.close

      sleep @period

    end

  end

  # escape html symbols (&/><, copied from cgi class)
  def escapeHTML( value) 
    return value.gsub(/&/n, '&amp;').gsub(/\"/n, '&quot;').gsub(/>/n, '&gt;').gsub(/</n, '&lt;')
  end

end
